/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SimpleBindingComponentServiceUnitManager.java 601 2006-06-13 12:53:50Z wjoseph $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.component.common.basic;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Generic service unit manager. Only handles one type of service unit.
 * Endpoints are activated by this manager. An abstract deploy method allows you
 * to add extra information in the service unit descriptor.
 * 
 * @version $Rev: 250 $ $Date: 2006-04-21 14:20:57 +0200 (ven, 21 avr 2006) $
 * @since Petals 1.0
 * @author wjoseph - eBMWebsourcing
 */
public class SimpleBindingComponentServiceUnitManager extends
        AbstractServiceUnitManager {

    protected static final String NO_ENDPOINT_FOR_CONSUMES_NODE = "No endpoint was found matching information from the consumes node of the JBI descriptor";

    protected Map<ServiceEndpoint, String> endpointAddress;
    
    protected Map<String, ServiceEndpoint > addressEndpoint;

    /**
     * Default contructor
     */
    public SimpleBindingComponentServiceUnitManager() {
        super();
        endpointAddress = new HashMap<ServiceEndpoint, String>();
        addressEndpoint = new HashMap<String, ServiceEndpoint>();
    }

    /**
     * Create a new SimpleBindingComponentServiceUnitManager
     * 
     * @param context
     *            Component's context
     * @param log
     *            Component's logger
     * @throws DeploymentException
     */
    public SimpleBindingComponentServiceUnitManager(ComponentContext context,
            Logger log) throws DeploymentException {
        super(context, log);
        endpointAddress = new HashMap<ServiceEndpoint, String>();
        addressEndpoint = new HashMap<String, ServiceEndpoint>();
    }

    /**
     * Returns the distant address for an endpoint. May be null if this endpoint
     * has no address registered for it
     * 
     * @param ep
     *            an Endpoint
     * @return the distant address for this endpoint, or null
     */
    public String getAddressForEndpoint(ServiceEndpoint ep) {
        return endpointAddress.get(ep);
    }
    
    /**
     * Return the endpoint associated with a distant address. May be null if
     * this address does not match any endpoint
     * 
     * @param address
     *            The distant address
     * @return the endpoint associated with the distant address, else null
     */
    public ServiceEndpoint getEndpointForAddress(String address) {
        return addressEndpoint.get(address);
    }

    /**
     * Same process as for AbstractServiceUnitManager except the registered
     * endpoint is stored with the distant address in a Map (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.basic.AbstractServiceUnitManager#start(java.lang.String)
     */
    @Override
    public void start(String serviceUnitName) throws DeploymentException {
        logger.log(Level.FINE, "start  serviceUnitName  " + serviceUnitName);
        String suRootPath = serviceUnitInstallationRootPath
                .get(serviceUnitName);
        Document serviceDesc = getServiceUnitWSDL(suRootPath);
        if (serviceDesc != null) {
            try {
                ServiceEndpoint ep = activateEndpointsFromJBIDescription(
                        serviceDesc, serviceUnitName, suRootPath);
                registerAddressForEndpoint(ep, suRootPath);
                registerEndpointForAddress(suRootPath);
            } catch (Exception ex) {
                logger.log(Level.SEVERE, FAILED_ACTIVATE_ENDPOINT + ex);
                throw new DeploymentException(FAILED_ACTIVATE_ENDPOINT, ex);
            }
        } else {
            throw new DeploymentException(INCOMPLETE_SERVICE_UNIT_PACKAGE);
        }
    }

    /**
     * The distant address registered for this endpoint is removed (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.basic.AbstractServiceUnitManager#stop(java.lang.String)
     */
    public void stop(String serviceUnitName) throws DeploymentException {
        try {
            ServiceEndpoint ep = serviceUnitMap.get(serviceUnitName);
            serviceUnitInstallationRootPath.remove(serviceUnitName);
            deactivateEndpointsFromJBIDescription(serviceUnitName);
            String address = endpointAddress.get(ep);
            if(address != null) {
                endpointAddress.remove(ep);
                addressEndpoint.remove(address);
            }
        } catch (Exception e) {
            throw new DeploymentException(FAILED_DEACTIVATE_ENDPOINT, e);
        }
        logger.log(Level.FINE, "stop  serviceUnitName  " + serviceUnitName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.petals.component.common.basic.AbstractServiceUnitManager#deploy(java.util.Map)
     */
    @Override
    protected void deploy(Map<String, String> extensions) {
    }

    /**
     * Register a distant address for the new endpoint. The address is found in
     * the jbiXml file as an extension element.
     * 
     * @param ep
     *            the new endpoint
     * @param suRootPath
     *            the service unit's root path
     */
    protected void registerAddressForEndpoint(ServiceEndpoint ep,
            String suRootPath) {
        File jbiXmlFile = new File(suRootPath + File.separator + "META-INF"
                + File.separator + "jbi.xml");
        Node providesNode = getNode(jbiXmlFile, "provides");
        Map<String, String> extensions = getExtensions(providesNode);
        endpointAddress.put(ep, extensions.get("address"));
    }
    
    /**
     * Register an endpoint for the distant address. The address is found in the
     * jbiXml file as an extension element. We search in the container if there
     * is an endpoint for the serviceName and endpointName found in the consumes
     * node. If no endpoint is found, we search for endpoints for the Interface
     * Name found in the consumes node. If still no endpoint is foud an
     * Exception is raised
     * 
     * @param suRootPath
     *            the service unit's root path
     * @throws Exception
     *             If bo endpoint is found for this serviceName or Interface
     *             Name
     */
    protected void registerEndpointForAddress(String suRootPath)
        throws Exception {
        File jbiXmlFile = new File(suRootPath + File.separator + "META-INF"
                + File.separator + "jbi.xml");
        Node consumesNode = getNode(jbiXmlFile, "consumes");
        Map<String, String> extensions = getExtensions(consumesNode);
        String address = extensions.get("address");
        if (address != null && consumesNode != null) {
            QName interfaceName = getInterfaceNameFromJbiXml(consumesNode);
            String endpointName = getEndpointNameFromJbiXml(consumesNode);
            QName serviceName = getServiceNameFromJbiXml(consumesNode);
            ServiceEndpoint endpoint = context.getEndpoint(serviceName,
                    endpointName);
            if (endpoint == null) {
                ServiceEndpoint[] endpoints = context
                        .getEndpoints(interfaceName);
                if (endpoints.length == 0) {
                    throw new Exception(NO_ENDPOINT_FOR_CONSUMES_NODE);
                } else {
                    addressEndpoint.put(address, endpoints[0]);
                }
            } else {
                addressEndpoint.put(address, endpoint);
            }
        }
    }
}
