/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.smtpbc.listeners;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Properties;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.mail.Message;
import javax.mail.NoSuchProviderException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.ebmwebsourcing.b2bgw.security.beans.CertificatePKCS12;
import org.objectweb.petals.binding.exception.SMTPBCException;

public class SenderMail {

    private X509Certificate caCert;

    private CertificatePKCS12 cert;

    private String certificatePath;

    private String from;

    private String host;

    private Logger logger;

    private Session session;

    private Transport trans;

    public SenderMail(String host, String from, String certificatePath,
        CertificatePKCS12 cert, Logger logger) {

        this.certificatePath = certificatePath;
        this.cert = cert;
        this.host = host;
        this.from = from;
        this.logger = logger;
        initialization();
    }

    public void initialization() {

        Properties props = new Properties();
        props.put("mail.smtp.host", this.host);
        session = Session.getDefaultInstance(props, null);
        session.setDebug(false);
        try {
            trans = session.getTransport("smtp");
        } catch (NoSuchProviderException e) {
            this.logger.log(Level.SEVERE, e.getMessage());
        }
        this.caCert = loadCACertificate();
    }

    public X509Certificate loadCACertificate() {
        CertificateFactory cf;
        try {
            cf = CertificateFactory.getInstance("X.509");
            // load autority certificate
            return (X509Certificate) cf
                .generateCertificate(new FileInputStream(certificatePath
                    + "ca.crt"));
        } catch (CertificateException e) {
            this.logger.log(Level.WARNING, "Impossible to load certificate "
                + e.getMessage());
        } catch (FileNotFoundException e) {
            this.logger.log(Level.WARNING,
                "Autority Certificate file not found " + e.getMessage());
        }
        return null;
    }

    public X509Certificate loadReceiverCertificate(String fileName)
        throws SMTPBCException {
        X509Certificate receiptCert = null;
        CertificateFactory cf;
        try {
            cf = CertificateFactory.getInstance("X.509");

            // load receiver certificate, to take public key
            receiptCert = (X509Certificate) cf
                .generateCertificate(new FileInputStream(certificatePath
                    + File.separator + fileName));
        } catch (CertificateException e) {
            throw new SMTPBCException(
                "Impossible to load receiver certificate ", e.getMessage());
        } catch (FileNotFoundException e) {
            throw new SMTPBCException("Receiver certificate file not found ", e
                .getMessage());
        }
        return receiptCert;
    }

    /**
     * "sendMail()" creates un mime message, and sends it to recipient "to"
     * 
     * @param type
     *            type is to send and simple message Mime or , un crypted et/or
     *            signed message
     * @param msgBody
     *            msgBody is the content of the message
     * @param to
     *            to is mail address of external service
     * @throws SMTPBCException
     * @throws MessagingException
     * @throws Exception
     */
    public void sendMail(String type, String msgBody, String to,
        String subject, Set<DataHandler> attachments) throws SMTPBCException {

        Message message = new MimeMessage(session);
        try {
            if (to != null) {
                message.setRecipient(javax.mail.Message.RecipientType.TO,
                    new InternetAddress(to));
            }

            message.setFrom(new InternetAddress(this.from));

            if (subject != null) {
                message.setSubject(subject);
            }
            if ("simple".equals(type)) {
                message = MessageMime.simpleMessage((MimeMessage) message,
                    msgBody, attachments);
            } else if ("signed".equals(type)) {
                // Create signed Mime message
                message = MessageMime.signedMultipart((MimeMessage) message,
                    this.cert, caCert, msgBody, attachments);

            } else if ("signedAndCrypted".equals(type)) {
                // Create signed and cryted Mime message, first signed , after
                // encryted message
                message = MessageMime.signedAndEnvelope((MimeMessage) message,
                    this.cert, this.caCert,
                    loadReceiverCertificate("externalServiceCertificate.csr"),
                    msgBody, attachments);
            } else {
                message = null;
            }
            // Send mail
            if (message != null) {
                transport(message);
            } else {
                throw new SMTPBCException(
                    "Message type isn't validate. Validate Types  : simple , signed , signedAndCrypted");
            }

        } catch (AddressException e) {
            throw new SMTPBCException("Error in address mail " + e.getMessage());
        } catch (javax.mail.MessagingException e) {
            throw new SMTPBCException(e.getMessage());
        }
    }

    public void transport(Message msg) throws SMTPBCException {
        try {
            this.trans.connect();
            msg.saveChanges();
            this.trans.sendMessage(msg, msg.getAllRecipients());
            this.trans.close();
            // Transport.send(msg);
        } catch (javax.mail.MessagingException e) {
            e.printStackTrace();
            // throw new SMTPBCException(e.getMessage());
        }
    }

}
