/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: IncomingMessageQueue.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.jbi.messaging;

import java.util.LinkedList;

import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Message queue attached to a
 * <code>Component</code> <code>DeliveryChannel</code>. Messages recevied
 * by the <code>Transporter</code> are stored in this queue until the
 * component accepts messages. TODO IncomingMessageQueue: stores messages
 * physicly , or manage the recevied messages differently: the Transporter keeps
 * all messages, and when an 'accept' method is called, ask the Transporter to
 * return a message, with the component name as parameter of the request. The
 * Transporter has to manage multi thread calls, and blocking calls. The
 * transporter has to manager message storing.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Gael BLONDELLE - EBM WebSourcing
 */
public class IncomingMessageQueue {

    /**
     * Monolog Logger instace.
     */
    protected LoggingUtil log;

    protected LinkedList<MessageExchangeImpl> messagesQueue;

    public IncomingMessageQueue(Logger logger) {
        log = new LoggingUtil(logger);

        messagesQueue = new LinkedList<MessageExchangeImpl>();
    }

    /**
     * This method wake up all threads that are waiting for a message
     * 
     */
    public void interrupt() {
        synchronized (messagesQueue) {
            messagesQueue.notifyAll();
        }
    }

    /**
     * Poll the oldest message. If there is no message, wait until a message is
     * pushed or timeoutMS exceeded This method is synchronized on its
     * <i>messagesQueue</i>
     * 
     * @return a message, or <b>null</b> after timeoutMS exceeded
     */
    public MessageExchangeImpl poll(long timeoutMS)
        throws IllegalArgumentException, InterruptedException {
        log.call(timeoutMS);

        MessageExchangeImpl mei = null;

        synchronized (messagesQueue) {
            if (!messagesQueue.isEmpty()) {
                // there is a message, return it
                mei = (MessageExchangeImpl) messagesQueue.removeFirst();
            } else {
                // there is no msg, wait
                long startTime = System.currentTimeMillis();

                messagesQueue.wait(timeoutMS);

                // check the reason of the end of the wait:
                // timeout or notify ->check the elapsed time

                long endTime = System.currentTimeMillis();

                boolean timeOutOccured = (endTime - startTime) >= timeoutMS;

                if (!messagesQueue.isEmpty()) {
                    mei = (MessageExchangeImpl) messagesQueue.removeFirst();
                } else if (!timeOutOccured) {
                    // The thread was wake up but there is no message,
                    // Assume that is because we are closing the exchange.
                    // Throw an interruptedException
                    throw new InterruptedException(
                        "The deliveryChannel is closing.");
                }
            }
        }
        return mei;
    }

    /**
     * Push <i>msg</i> in the queue and notify pop requesters. This method is
     * synchronized on its <i>messagesQueue</i>
     * 
     * @param msg
     */
    public void push(MessageExchangeImpl msg) {
        log.call(msg);

        synchronized (messagesQueue) {
            messagesQueue.addLast(msg);

            messagesQueue.notify();
        }
    }
}
