/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RepositoryService.java 1006 2006-09-07 13:06:40Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.repository;

import java.io.File;
import java.net.URI;

import org.objectweb.petals.PetalsException;

/**
 * Platform Component Repository.
 * 
 * @version $Rev: 1006 $ $Date: 2006-09-07 15:06:40 +0200 (jeu., 07 sept. 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public interface RepositoryService {

    /**
     * 
     * @param componentID
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @param zipArchiveLocation
     *            <code>URI</code> the component Zip archive file to add
     *            flatten into the repository.
     * @return the component installation root {@link URI}
     * @throws PetalsException
     *             if it fails to create install and work dirs for the component
     *             or if it fails to explode archive into install dir
     */
    URI addComponentPackage(String componentID, URI zipArchiveLocation)
        throws PetalsException;

    /**
     * 
     * @param sharedLibID
     *            <code>String</code> unique sl id that identifies an unique
     *            sl directory into the petals repository.
     * @param zipArchiveLocation
     *            <code>URI</code> the sl Zip archive file to add flatten into
     *            the repository.
     * @return the sl installation root {@link URI}
     * @throws PetalsException
     *             if it fails to create install and work dirs for the sl or if
     *             it fails to explode archive into install dir
     */
    URI addSharedLibPackage(String sharedLibID, URI zipArchiveLocation)
        throws PetalsException;

    /**
     * 
     * @param serviceAssemblyID
     *            <code>String</code> unique sa id that identifies an unique
     *            sa directory into the petals repository.
     * @param zipArchiveLocation
     *            <code>URI</code> the sa Zip archive file to add flatten into
     *            the repository.
     * @return the sa installation root {@link URI}
     * @throws PetalsException
     *             if it fails to create install and work dirs for the sa or if
     *             it fails to explode archive into install dir
     */
    URI addServiceAssemblyPackage(String serviceAssemblyID,
            URI zipArchiveLocation) throws PetalsException;

    /**
     * Verifies if the Petals repository contains a component directory with the
     * specified identification.
     * 
     * @param compId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @return <code>true</code> if that directory exists. Otherwise, returns
     *         <code>false</code>.
     */
    boolean containsComponentPackage(String compId);

    /**
     * Verifies if the Petals repository contains a sa directory with the
     * specified identification.
     * 
     * @param saId
     *            <code>String</code> unique sa id that identifies an unique
     *            sa directory into the petals repository.
     * @return <code>true</code> if that directory exists. Otherwise, returns
     *         <code>false</code>.
     */
    boolean containsServiceAssemblyPackage(String saId);

    /**
     * Verifies if the Petals repository contains a sl directory with the
     * specified identification.
     * 
     * @param slId
     *            <code>String</code> unique sl id that identifies an unique
     *            sl directory into the petals repository.
     * @return <code>true</code> if that directory exists. Otherwise, returns
     *         <code>false</code>.
     */
    boolean containsSharedLibPackage(String slId);

    /**
     * 
     * @param suId
     *            <code>String</code> unique SU identifier within the
     *            repository.
     * @param suZipLocation
     *            <code>URI</code> the SU Zip archive file to add flatten into
     *            the repository.
     * @param saId
     *            <code>String</code> unique SA identifier within the
     *            repository where SU add to be flatten
     * @return <code>URI</code> installation root of su in the repository.
     *         Must be a valid directory.
     * @throws PetalsException
     */
    URI explodeSUIntoSAInstallFolder(String suId, URI suZipLocation, String saId)
        throws PetalsException;

    /**
     * Returns the install directory of the component
     * 
     * @param componentId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @return the install directory of the component
     */
    File getComponentInstallRoot(String componentId);

    /**
     * Returns the root directory of this component
     * 
     * @param componentId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @return the root directory of this component
     */
    File getComponentRoot(String componentId);

    /**
     * Returns the work directory of the component
     * 
     * @param componentId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @return the work directory of the component
     */
    File getComponentWorkRoot(String componentId);

    /**
     * Returns the root directory of this sl
     * 
     * @param slId
     *            <code>String</code> unique sl id that identifies an unique
     *            sl directory into the petals repository.
     * @return the root directory of this sl
     */
    File getSharedLibRoot(String slId);

    /**
     * Returns the work directory of the sl
     * 
     * @param slId
     *            <code>String</code> unique sl id that identifies an unique
     *            sl directory into the petals repository.
     * @return the work directory of the sl
     */
    File getSharedLibWorkRoot(String slId);

    /**
     * Returns the install directory of the sl
     * 
     * @param slId
     *            <code>String</code> unique sl id that identifies an unique
     *            sl directory into the petals repository.
     * @return the install directory of the sl
     */
    File getSharedLibInstallRoot(String slId);

    /**
     * Returns the root directory of this sa
     * 
     * @param saId
     *            <code>String</code> unique sa id that identifies an unique
     *            sa directory into the petals repository.
     * @return the root directory of this sa
     */
    File getServiceAssemblyRoot(String saId);

    /**
     * Returns the work directory of the sa
     * 
     * @param saId
     *            <code>String</code> unique sa id that identifies an unique
     *            sa directory into the petals repository.
     * @return the work directory of the sa
     */
    File getServiceAssemblyWorkRoot(String saId);

    /**
     * Returns the install directory of the sa
     * 
     * @param saId
     *            <code>String</code> unique sa id that identifies an unique
     *            sa directory into the petals repository.
     * @return the install directory of the sa
     */
    File getServiceAssemblyInstallRoot(String saId);

    /**
     * Delete and clean all directories and related data from the Petals
     * repository, for the given component identifier.
     * 
     * @param componentId
     *            <code>String</code> unique component id that identifies an
     *            unique component directory into the petals repository.
     * @return <code>true</code> for sucessful removal. Otherwise,
     *         <code>false</code>.
     */
    boolean removeComponentPackage(String componentId);

    /**
     * Delete and clean all directories and related data from the Petals
     * repository, for the given sl identifier.
     * 
     * @param slId
     *            <code>String</code> unique sl id that identifies an unique
     *            sl directory into the petals repository.
     * @return <code>true</code> for sucessful removal. Otherwise,
     *         <code>false</code>.
     */
    boolean removeSharedLibPackage(String slId);

    /**
     * Delete and clean all directories and related data from the Petals
     * repository, for the given sa identifier.
     * 
     * @param saId
     *            <code>String</code> unique sa id that identifies an unique
     *            sa directory into the petals repository.
     * @return <code>true</code> for sucessful removal. Otherwise,
     *         <code>false</code>.
     */
    boolean removeServiceAssemblyPackage(String saId);

    /**
     * Retrieve the components directory absolute path from the Petals
     * repository. This directory contains all installed components.
     * 
     * @return the components directory URL from the Petals repository
     */
    String getComponentsDirectory();

    /**
     * Retrieve the ServiceAssemblies directory absolute path from the Petals
     * repository. This directory contains all deployed ServiceAssemblies.
     * 
     * @return the ServiceAssemblies directory URL from the Petals repository
     */
    String getServiceAssembliesDirectory();

    /**
     * Retrieve the SharedLibs directory absolute path from the Petals
     * repository. This directory contains all installed SharedLibs.
     * 
     * @return the SharedLibs directory URL from the Petals repository
     */
    String getSharedLibsDirectory();
    
    /**
     * Retrieve the "lost+found" directory absolute path from the Petals
     * repository. This directory contains all unknown SharedLibs, Components and ServiceAssemblies.
     * 
     * @return the SharedLibs directory URL from the Petals repository
     */
    String getLostPlusFoundDirectory();
}
