/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIDescriptorBuilderTest.java 632 2006-06-22 07:28:26Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbicommon.descriptor;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;

import org.easymock.classextension.EasyMock;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * @version $Rev: 632 $ $Date: 2006-06-22 07:28:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * @author ddesjardins - eBMWebsourcing
 */
public class JBIDescriptorBuilderTest extends TestCase {

    private String baseDir;

    public void setUp() {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
    }

    /**
     * Testing load and parse operation on descriptors files listed in testing
     * resource data directory.
     * 
     * @throws Exception
     */
    public final void testLoadingDescriptors() throws Exception {
        File[] descriptors = listDescritorsTestingData();
        if (descriptors != null) {
            for (File aFile : descriptors) {
                new JBIDescriptorBuilder(aFile.toURI(), null).parseJbiXml(aFile
                    .toURI());
            }
        }
    }

    /**
     * Testing validation using the JBI Descriptor XML Schema on descriptors
     * files listed in testing resource data directory.
     * 
     * @throws Exception
     */
    public final void testValidatingDescriptors() throws Exception {
        File[] descriptors = listDescritorsTestingData();
        if (descriptors != null) {
            for (File aFile : descriptors) {
                URI sourceURI = aFile.toURI();
                assertTrue(new JBIDescriptorBuilder(sourceURI, null)
                    .validateJbiXml(sourceURI));
            }
        }
    }

    /**
     * Used to retrieve the list of descriptor files
     * 
     * @return list of descriptor file
     * @throws Exception
     */
    private File[] listDescritorsTestingData() throws Exception {
        File dir = new File(baseDir
            + File.separator + "descriptors");
        if (dir == null
            || !dir.exists()) {
            return null;
        }
        return dir.listFiles();
    }

    public void testGetJbiDescriptorAsString() {
        String baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        JBIDescriptorBuilder builder = new JBIDescriptorBuilder(new File(
            baseDir
                + "src" + File.separator + "test-data" + File.separator
                + "descriptors" + File.separator + "emptyDescriptor.xml")
            .toURI(), null);
        assertEquals(builder.getJbiDescriptorAsString(), "<jbi />");
        assertEquals(builder.getURI(), new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "descriptors" + File.separator + "emptyDescriptor.xml").toURI());
    }

    public void testJbiXmlErrorHandler() {
        Logger logger = EasyMock.createMock(Logger.class);

        JBIDescriptorBuilder.JbiXmlErrorHandler handler = new JBIDescriptorBuilder.JbiXmlErrorHandler(
            logger);

        Locator locator = EasyMock.createMock(Locator.class);

        EasyMock.expect(locator.getPublicId()).andReturn("1");
        EasyMock.expect(locator.getSystemId()).andReturn("1");
        EasyMock.expect(locator.getLineNumber()).andReturn(10);
        EasyMock.expect(locator.getColumnNumber()).andReturn(10);

        EasyMock.replay(locator);
        SAXParseException saException = new SAXParseException("test", locator);

        logger.log(Level.SEVERE, "test", saException);
        logger.log(Level.SEVERE, "test", saException);
        logger.log(Level.WARNING, "test", saException);

        EasyMock.replay(logger);
        try {
            handler.error(saException);
        } catch (SAXException e) {
            // do nothing
        }

        try {
            handler.fatalError(saException);
        } catch (SAXException e) {
            // do nothing
        }

        try {
            handler.warning(saException);
        } catch (SAXException e) {
            // do nothing
        }
    }

    public void testConstructorException() {
        try {
            new JBIDescriptorBuilder(null, null);
            fail();
        } catch (Exception e) {
            // do nothing
        }
    }

    public void testExtractXmlAttributeQName() throws SecurityException,
        NoSuchMethodException {
        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class,
            new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
                "buildDescriptor", new Class[] {Document.class})});

        Node node = EasyMock.createMock(Node.class);
        NamedNodeMap nodeMap = EasyMock.createMock(NamedNodeMap.class);
        Node outNode = EasyMock.createMock(Node.class);

        EasyMock.expect(node.getAttributes()).andReturn(nodeMap);
        EasyMock.expect(nodeMap.getNamedItem("test")).andReturn(outNode);
        EasyMock.expect(outNode.getNodeValue()).andReturn("out");

        EasyMock.replay(nodeMap);
        EasyMock.replay(outNode);
        EasyMock.replay(node);

        assertEquals(QName.valueOf("out"), builder.extractXmlAttributeQName(
            node, "test"));
    }

    public void testExtractXmlAttributeQNameWithNS() throws SecurityException,
        NoSuchMethodException {
        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class,
            new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
                "buildDescriptor", new Class[] {Document.class})});

        Node node = EasyMock.createMock(Node.class);
        NamedNodeMap nodeMap = EasyMock.createMock(NamedNodeMap.class);
        Node outNode = EasyMock.createMock(Node.class);
        Node nsNode = EasyMock.createMock(Node.class);

        EasyMock.expect(node.getAttributes()).andReturn(nodeMap).anyTimes();
        EasyMock.expect(nodeMap.getNamedItem("test")).andReturn(outNode);
        EasyMock.expect(outNode.getNodeValue()).andReturn("ns:out");
        EasyMock.expect(node.getParentNode()).andReturn(null).anyTimes();
        EasyMock.expect(nodeMap.getLength()).andReturn(1);
        EasyMock.expect(nodeMap.item(0)).andReturn(nsNode);
        EasyMock.expect(nsNode.getNodeName()).andReturn("xmlns:ns");
        EasyMock.expect(nsNode.getNodeValue()).andReturn("outvalue");

        EasyMock.replay(nsNode);
        EasyMock.replay(nodeMap);
        EasyMock.replay(outNode);
        EasyMock.replay(node);

        assertEquals(QName.valueOf("{outvalue}out"), builder
            .extractXmlAttributeQName(node, "test"));
    }

    public void testBuild() throws SecurityException, NoSuchMethodException,
        SAXException, ParserConfigurationException, IOException {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = factory.newDocumentBuilder();
        File xml = new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "descriptors" + File.separator + "emptyDescriptor.xml");
        Document document = documentBuilder.parse(xml);

        JBIDescriptor jbiDescriptor = new JBIDescriptor();

        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class, new Method[] {
                JBIDescriptorBuilder.class.getDeclaredMethod("validateJbiXml",
                    new Class[] {URI.class}),
                JBIDescriptorBuilder.class.getDeclaredMethod("parseJbiXml",
                    new Class[] {URI.class}),
                JBIDescriptorBuilder.class.getDeclaredMethod("buildDescriptor",
                    new Class[] {Document.class})});

        EasyMock.expect(builder.validateJbiXml(new File(baseDir
            + "target" + File.separator + "test.xml").toURI())).andReturn(
            Boolean.TRUE);
        EasyMock.expect(builder.parseJbiXml(new File(baseDir
            + "target" + File.separator + "test.xml").toURI())).andReturn(
            document);
        EasyMock.expect(builder.buildDescriptor(document)).andReturn(
            jbiDescriptor);

        EasyMock.replay(builder);

        builder.jbiXmlUri = new File(baseDir
            + "target" + File.separator + "test.xml").toURI();

        assertEquals(builder.build(), jbiDescriptor);
        EasyMock.verify(builder);
    }

    public void testBuildException() throws SecurityException,
        NoSuchMethodException, SAXException, ParserConfigurationException,
        IOException {
        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class, new Method[] {
                JBIDescriptorBuilder.class.getDeclaredMethod("validateJbiXml",
                    new Class[] {URI.class}),
                JBIDescriptorBuilder.class.getDeclaredMethod("parseJbiXml",
                    new Class[] {URI.class}),
                JBIDescriptorBuilder.class.getDeclaredMethod("buildDescriptor",
                    new Class[] {Document.class})});
        Logger logger = EasyMock.createMock(Logger.class);

        IOException ioException = new IOException();

        EasyMock.expect(builder.validateJbiXml(new File(baseDir
            + "target" + File.separator + "test.xml").toURI())).andThrow(
            ioException);
        logger.log(Level.SEVERE, null, ioException);

        EasyMock.replay(builder);
        EasyMock.replay(logger);

        builder.log = logger;
        builder.jbiXmlUri = new File(baseDir
            + "target" + File.separator + "test.xml").toURI();

        try {
            builder.build();
            fail();
        } catch (SAXException e) {
            // do nothing
        }
        EasyMock.verify(builder);
        EasyMock.verify(logger);
    }

    public void testParseJbiXml() throws SecurityException,
        NoSuchMethodException, SAXException, IOException {
        File jbiFile = new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "descriptors" + File.separator + "emptyDescriptor.xml");

        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class, new Method[] {
                JBIDescriptorBuilder.class.getDeclaredMethod("validateJbiXml",
                    new Class[] {URI.class}),
                JBIDescriptorBuilder.class.getDeclaredMethod("buildDescriptor",
                    new Class[] {Document.class})});

        assertNotNull(builder.parseJbiXml(jbiFile.toURI()));
    }

    public void testValidateJbiXml() throws SecurityException,
        NoSuchMethodException, SAXException, IOException {
        File jbiFile = new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "descriptors" + File.separator
            + "componentInstallationDescriptor.xml");

        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class,
            new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
                "buildDescriptor", new Class[] {Document.class})});
        Logger logger = EasyMock.createMock(Logger.class);

        EasyMock.replay(logger);

        builder.log = logger;

        assertTrue(builder.validateJbiXml(jbiFile.toURI()));
    }

    public void testValidateJbiXmlException() throws SecurityException,
        NoSuchMethodException, SAXException, IOException {
        File jbiFile = new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "descriptors" + File.separator + "badDescriptor.xml");

        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class,
            new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
                "buildDescriptor", new Class[] {Document.class})});
        Logger logger = EasyMock.createMock(Logger.class);

        builder.log = logger;

        assertFalse(builder.validateJbiXml(jbiFile.toURI()));
    }

    public void testbuildDescriptor() throws SecurityException,
        NoSuchMethodException, SAXException, IOException,
        ParserConfigurationException {

        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class,
            new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
                "validateJbiXml", new Class[] {URI.class})});

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = factory.newDocumentBuilder();
        File xml = new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "descriptors" + File.separator
            + "componentInstallationDescriptor.xml");
        Document document = documentBuilder.parse(xml);

        JBIDescriptor descriptor = builder.buildDescriptor(document);
        assertNotNull(descriptor.component);
    }

    public void testbuildDescriptor1() throws SecurityException,
        NoSuchMethodException, SAXException, IOException,
        ParserConfigurationException {

        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class,
            new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
                "validateJbiXml", new Class[] {URI.class})});

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = factory.newDocumentBuilder();
        File xml = new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "descriptors" + File.separator
            + "serviceAssemblyDeploymentDescriptor.xml");
        Document document = documentBuilder.parse(xml);

        JBIDescriptor descriptor = builder.buildDescriptor(document);
        assertNotNull(descriptor.serviceAssembly);
    }

    public void testbuildDescriptor2() throws SecurityException,
        NoSuchMethodException, SAXException, IOException,
        ParserConfigurationException {

        JBIDescriptorBuilder builder = EasyMock.createMock(
            JBIDescriptorBuilder.class,
            new Method[] {JBIDescriptorBuilder.class.getDeclaredMethod(
                "validateJbiXml", new Class[] {URI.class})});

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = factory.newDocumentBuilder();
        File xml = new File(baseDir
            + "src" + File.separator + "test-data" + File.separator
            + "descriptors" + File.separator
            + "sharedLibraryInstallationDescriptor.xml");
        Document document = documentBuilder.parse(xml);

        JBIDescriptor descriptor = builder.buildDescriptor(document);
        assertNotNull(descriptor.sharedLibrary);
    }
}
